//@ts-nocheck
import { Vue, Component } from 'vue-property-decorator';
import { Getter, Mutation } from 'vuex-class';
import { types } from '@/common/store/index';
@Component({})
export default class CanvasMixin extends Vue {
	async getQr(data, fun): Promise<any> {
		// 获取分享小程序码
		// let name = data.type + data.share + data.uid;
		let name = +new Date();
		console.log('分享码带的参数', data);
		// let sceneStr = '';
		// for (let key in data.scene) {
		// 	sceneStr = sceneStr + key + '=' + data.scene[key] + '&';
		// }
		await this.$http
			.post({
				url: this.$api.Qrcode,
				data: { page: data.page, scene: data.scene }
			})
			.then(res => {
				if (!res.url) {
					wx.showToast({
						title: '小程序码生成失败!',
						icon: 'none'
					});
					return;
				}
				this.base64src(
					res.url,
					miniImage => {
						fun(miniImage);
					},
					name
				);
			})
			.catch(err => {
				wx.hideLoading();
			});
	}
	//远程图片转临时文件
	loadFile(img: string, fun) {
		uni.downloadFile({
			url: img,
			success(result) {
				if (result.statusCode === 200) {
					fun(result.tempFilePath);
				}
			}
		});
	}
	// base64图片 转成临时文件 传个name 保证生成的临时文件名唯一
	base64src(base64data, cb, name): any {
		let fsm = uni.getFileSystemManager();
		let FILE_BASE_NAME = 'tmp_base64src' + name; // 自定义文件名
		const [, format, bodyData] = /data:image\/(\w+);base64,(.*)/.exec(base64data) || [];
		if (!format) {
			return new Error('ERROR_BASE64SRC_PARSE');
		}
		const filePath = `${wx.env.USER_DATA_PATH}/${FILE_BASE_NAME}.${format}`;
		const buffer = uni.base64ToArrayBuffer(bodyData);
		fsm.writeFile({
			filePath,
			data: buffer,
			encoding: 'binary',
			success() {
				console.log(5555, filePath);
				cb(filePath);
			},
			fail(err) {
				console.log(66666, err);
				return new Error('ERROR_BASE64SRC_WRITE');
			}
		});
	}
	// 文字省略号
	ellipsisText(
		ctx,
		content, // 文字内容
		textWidth, // 最大宽度
		x, // 起点横坐标
		y, // 起点纵坐标
		color, // 文字颜色
		size, // 文字大小
		lineHeight, // 行高，针对多行
		rowNum = 1, // 行数
		weight = 'normal ' // 文字粗细
	): void | boolean {
		//textBaseline设为top，使文字的位置以左上角为基准
		// 将传入的文字内容（字符串）分割成单个字符串数组 'text' => ['t','e','x','t']
		if (!content) {
			return false;
		}
		let contentArr: string = content.toString().split('');
		let tempStr: string = ''; // 用来存放临时拼接的字符串
		let resultArr: string[] = []; // 最终生成的字符串数组
		// 将分割的单个字符拼成textWidth宽度的数组
		for (let i = 0; i < contentArr.length; i++) {
			// ctx.measureText => 返回文本宽度
			if (ctx.measureText(tempStr).width < textWidth) {
				// 当 拼接字符串宽度小于传入的宽度时，继续拼接
				tempStr += contentArr[i];
			} else {
				// 当拼接字符串宽度大于等于传入的宽度时，将字符串放入数组
				i--; // 防止当前参与循环的字符丢失
				resultArr.push(tempStr); // 将字符串放入数组
				tempStr = ''; // 拼接字符串清空，继续循环
			}
		}
		// 结束循环后，将最后一个拼接字符串放入数组
		resultArr.push(tempStr);

		// 截取传入行数的字符串数组
		/**
		 * slice(start,end) 方法可从已有的数组中返回选定的元素。
		 * start => 从何处开始选取。如果是负数，那么从数组尾部开始算起。
		 * end => 是数组片断结束处的数组下标。默认从 start 到数组结束的所有元素。
		 *        如果这个参数是负数，那么从数组尾部开始算起
		 */
		if (resultArr.length > rowNum) {
			let sliceArr = resultArr.slice(0, rowNum);
			// 判断截取数组的最后一个元素宽度是否和传入宽度相等，若相等，拼接"..."
			let eslipStr = sliceArr[rowNum - 1];
			if (ctx.measureText(eslipStr).width >= textWidth) {
				eslipStr += '...';
				// 将数组最后一个元素替换为拼接了'...'的字符串
				/**
				 * splice() 方法可删除从 index 处开始的零个或多个元素，
				 * 并且用参数列表中声明的一个或多个值来替换那些被删除的元素
				 * 返回的是含有被删除的元素的数组
				 * 会直接对数组进行修改
				 */
				sliceArr.splice(rowNum - 1, 1, eslipStr);
			}
			resultArr = sliceArr;
		}
		/**
		 * 绘制文字
		 */
		ctx.textBaseline = 'top';
		ctx.setFillStyle(color);
		ctx.font = weight + size + ' Arial, Helvetica, sans-serif';
		for (let index = 0; index < resultArr.length; index++) {
			ctx.fillText(resultArr[index], x, y + index * lineHeight);
		}
	}
	//画四边形
	lineSet(
		ctx,
		x: number,
		y: number,
		w: number,
		h: number,
	) {
		// ctx,起点x坐标，起点y坐标，宽度，高度
		ctx.beginPath();
		ctx.strokeStyle = "#F6F6F6";
		ctx.rect(x, y, w, h);
		ctx.stroke();
	}
	// 绘制不规则圆角矩形 (任一圆角矩形)
	roundRect(
		ctx,
		x: number,
		y: number,
		w: number,
		h: number,
		color: string,
		r1 = 0,
		r2 = 0,
		r3 = 0,
		r4 = 0
	) {
		// ctx,起点x坐标，起点y坐标，宽度，高度，圆角半径，颜色
		ctx.beginPath();
		ctx.moveTo(x + r4, y); // 从左上角横线起点开始画
		ctx.lineTo(x + w - r1, y);
		ctx.arcTo(x + w, y, x + w, y + r1, r1);
		ctx.lineTo(x + w, y + h - r2);
		ctx.arcTo(x + w, y + h, x + w - r2, y + h, r2);
		ctx.lineTo(x + r3, y + h);
		ctx.arcTo(x, y + h, x, h + y - r3, r3);
		ctx.lineTo(x, y + r4);
		ctx.arcTo(x, y, x + r4, y, r4);
		ctx.closePath();
		ctx.setFillStyle(color);
		ctx.fill();
	}
	textSet(ctx, text, size, x, y, color, weight = 'normal ') {
		ctx.setFillStyle(color);
		ctx.font = weight + size + ' Arial, Helvetica, sans-serif';
		ctx.fillText(text, x, y);
	}
	textCenterSet(ctx, text, size, x, y, color, maxWidth, weight = 'normal ') {
		ctx.setFillStyle(color);
		ctx.font = weight + size + ' Arial, Helvetica, sans-serif';
		ctx.textAlign = "center";
		ctx.fillText(text, x, y, maxWidth);
	}
	// 裁剪图片居中
	drawImgInfo(ctx, dx, dy, dWidth, dHeight, head_img, imgX, imgY) {
		if (!head_img) return;
		// 裁剪图片居中，ctx, 位置，画布的宽高，图片,图片的实际宽高
		let sx, sy, sWidth, sHeight;
		if (imgX > imgY) {
			//获取图片（裁剪）后的实际宽高，即取图片最短边作为原始边长
			sWidth = (imgY * dWidth) / dHeight;
			sHeight = imgY;
			sx = (imgX - sWidth) / 2;
			sy = 0;
		} else {
			sWidth = imgX;
			sHeight = (imgX * dHeight) / dWidth;
			sx = 0;
			sy = (imgY - sHeight) / 2;
		}
		ctx.drawImage(head_img, sx, sy, sWidth, sHeight, dx, dy, dWidth, dHeight);
	}
	// 绘制圆角矩形图片
	roundImg(ctx, x, y, w, h, r, color, url) {
		// ctx,起点x坐标，起点y坐标，宽度，高度，圆角半径，颜色，图片地址
		ctx.save(); // 保存当前环境的状态.  ctx.save();ctx.restore()避免clip对全局造成影响
		ctx.beginPath();
		ctx.moveTo(x + r, y); // 从左上角横线起点开始画
		ctx.lineTo(x + w - r, y);
		ctx.arcTo(x + w, y, x + w, y + r, r);
		ctx.lineTo(x + w, y + h - r);
		ctx.arcTo(x + w, y + h, x + w - r, y + h, r);
		ctx.lineTo(x + r, y + h);
		ctx.arcTo(x, y + h, x, h + y - r, r);
		ctx.lineTo(x, y + r);
		ctx.arcTo(x, y, x + r, y, r);
		ctx.closePath();
		ctx.clip(); // 从原始画布剪切任意形状和尺寸的区域
		ctx.drawImage(url, x, y, w, h); // 向画布上绘制图像、画布或视频
		ctx.restore(); // 返回之前保存过的路径状态和属性
		console.log(222);
	}
	// 绘制圆角图片
	drawCircular(ctx, width, height, x, y, url) {
		if (!url) return;
		ctx.save(); // 保存当前环境的状态
		ctx.beginPath(); // 起始一条路径，或重置当前路径
		/**
		 * ctx.arc(x,y,r,sAngle,eAngle,counterclockwise);
		 * x：圆的中心的 x 坐标。
		 * y：圆的中心的 y 坐标。
		 * r：圆的半径。
		 * sAngle：起始角，以弧度计。（弧的圆形的三点钟位置是 0 度）。
		 * eAngle：结束角，以弧度计。
		 * counterclockwise：可选，规定应该逆时针还是顺时针绘图。False = 顺时针，true = 逆时针。
		 */
		ctx.arc(width / 2 + x, height / 2 + y, width / 2, 0, Math.PI * 2, false); // 创建弧/曲线（用于创建圆形或部分圆）
		ctx.clip(); // 从原始画布剪切任意形状和尺寸的区域
		ctx.drawImage(url, x, y, width, height); // 向画布上绘制图像、画布或视频
		ctx.restore(); // 返回之前保存过的路径状态和属性
	}
	// 保存分享图片
	saveImageToPhotos(id): void {
		const that = this;
		console.log(2222222);
		uni.showLoading({
			title: '生成图片中...'
		});
		/**
		 * uni.canvasToTempFilePath(Object object, Object this)
		 * 把当前画布指定区域的内容导出生成指定大小的图片。
		 * 在 draw() 回调里调用该方法才能保证图片导出成功。
		 * canvasId => 画布标识，传入 canvas 组件的 canvas-id
		 * 使用自定义组件调取 uni.canvasToTempFilePath 接口获取不到canvas，同样的需要传入上下文的this
		 */
		uni.canvasToTempFilePath(
			{
				canvasId: id,
				success(res) {
					console.log('canvasTemppath', res);
					// tempFilePath => 生成文件的临时路径 (本地路径)
					const canvasTemppath: string = res.tempFilePath;
					uni.hideLoading();
					/**
					 * uni.saveImageToPhotosAlbum(Object object)
					 * 保存图片到系统相册。
					 * 调用前需要 用户授权 scope.writePhotosAlbum
					 * filePath => 图片文件路径，可以是临时文件路径或永久文件路径 (本地路径) ，不支持网络路径
					 */
					uni.saveImageToPhotosAlbum({
						filePath: canvasTemppath,
						success(res) {
							uni.showToast({
								title: '保存成功',
								icon: 'success',
								duration: 1500
							});
						},
						fail(res) {
							console.error('saveImageToPhotosAlbum-err', res);
							if (
								res.errMsg === 'saveImageToPhotosAlbum:fail:auth denied' ||
								res.errMsg === 'saveImageToPhotosAlbum:fail auth deny' ||
								res.errMsg === 'saveImageToPhotosAlbum:fail authorize no response'
							) {
								uni.showModal({
									title: '提示',
									content: '需要您授权保存相册',
									showCancel: false,
									success(res) {
										/**
										 * uni.openSetting(Object object)
										 * 调起客户端小程序设置界面，返回用户设置的操作结果。
										 * 设置界面只会出现小程序已经向用户请求过的权限。
										 * withSubscriptions => 是否同时获取用户订阅消息的订阅状态，默认不获取。
										 * withSubscriptions 只返回用户勾选过订阅面板中的“总是保持以上选择，不再询问”的订阅消息。
										 * "scope.writePhotosAlbum" => 保存到相册的权限
										 */
										uni.openSetting({
											success(settingdata) {
												if (
													settingdata.authSetting[
													'scope.writePhotosAlbum'
													]
												) {
													uni.showModal({
														title: '提示',
														content: '授权成功,再次点击图片即可保存',
														showCancel: false
													});
												} else {
													uni.showModal({
														title: '提示',
														content: '获取权限失败，无法保存到相册哦~',
														showCancel: false
													});
												}
											},
											fail(res) {
												console.error('openSetting-err', res);
											}
										});
									}
								});
							} else {
								uni.showToast({
									title: '保存失败，请稍后重试~',
									icon: 'none',
									duration: 1500
								});
							}
						}
					});
				},
				fail(res) {
					console.log('canvasToTempFilePath-fail', res);
					if (res.errMsg === 'saveImageToPhotosAlbum:fail cancel') {
						uni.showToast({
							title: '取消保存',
							icon: 'none',
							duration: 1500
						});
					} else {
						uni.showToast({
							title: '保存失败，请稍后重试',
							icon: 'none',
							duration: 1500
						});
					}
				}
			},
			this
		);
	}
}