import Vue from 'vue';
import { RouterMount, createRouter } from 'uni-simple-router';
import Store, { types } from '@/common/store/index.ts';
import { $http } from '@/common/http/index.ts';
import $api from '@/common/api.ts';
import errorPage from '@/pages/error/404/404.vue';

var time = 0
const router = createRouter({
	encodeURI: false,
	platform: process.env.VUE_APP_PLATFORM,
	routerErrorEach: ({ type, level, ...args }) => {
		// #ifdef APP-PLUS
		if (type === 3) {
			router.$lockStatus = false;
			time++
			if (time == 2) {
				plus.runtime.quit();
			} else {
				uni.showToast({
					title: '再按一次退出应用',
					icon: 'none'
				})
			}
			setTimeout(() => {
				time = 0
			}, 1500)
		}
		// #endif
	},
	routes: [
		...ROUTES,
		{
			path: '*',
			component: errorPage
		}
	]
});
//路由守卫
router.beforeEach(async (to: any, from: any, next: any) => {
	// console.log(from, 1231231232);
	if (to.meta.pay) {
		next();
		return;
	}
	let flag = false;
	let referrer = getSuperior(from, to);
	//首页或者没有token 去登录
	if (!(Store.state as any).base.token || to.name === 'front') {
		await login(referrer || 0, to, 'login');
		flag = true;
	}
	//非首页没有token 去注册
	if (to.name !== 'front' && !(Store.state as any).base.token) {
		await login(referrer || 0, to, 'register');
		flag = true;
	}
	//首页没有token 存referrer
	if (to.name == 'front' && !(Store.state as any).base.token) {
		referrer && uni.setStorageSync('referrer', referrer);
	}
	// 有referrer 登录状态 绑上级
	if (!flag && (Store.state as any).base.token) {
		referrer && $http.put({
			url: $api.bindReferrer,
			data: {
				referrer: +referrer
			}
		})
	}
	if (await isAuthorization(to, from, next)) return;
	next();
});
function getSuperior(from: any, to: any): any {
	if (from.query.isShare === '1' && from.query.uid) {
		return +from.query.uid;
	}
	if (to.query.scene) {
		let str = decodeURIComponent(to.query.scene);
		if (str.indexOf("&") > -1) {
			let sceneArr = decodeURIComponent(to.query.scene).split('&');
			let scene: any = {};
			sceneArr.forEach((item: any) => {
				let arr = item.split('=');
				if (arr[0]) scene[arr[0]] = arr[1];
			})
			if (scene.isShare === '1' && scene.uid) {
				return +scene.uid;
			}
		} else {
			let sceneArr = decodeURIComponent(to.query.scene).split(',');
			if (sceneArr[0] === 'isShare' && sceneArr[1]) {
				return +sceneArr[1];
			}
		}
	}
	return null;
}
//登录 注册
function login(superior: any, to: any, api: any) {
	return new Promise((resolve, reject) => {
		uni.login({
			provider: 'weixin',
			success: function(loginRes) {
				if (api === 'register') {
					let referrer = uni.getStorageSync('referrer');
					+superior || (superior = referrer);
				}
				$http.post({
					url: ($api as any)[api],
					data: { third: { code: loginRes.code, referrer: +superior || 0 }, auth: { sid: 0, agent: 1, way: 3, } }
				}).then((res: any) => {
					if (api === 'register') {
						uni.setStorageSync('referrer', '');
					}
					Store.commit(`base/${types.SET_TOKEN}`, res.token);
					if (res.registered) {
						uni.setStorageSync('referrer', '');
						$http.get({
							url: $api.getUserInfo,
						}).then((result: any) => {
							if (result) {
								Store.commit(`base/${types.SET_USERINFO}`, result.info);
							}
						}, (err: any) => {
							console.log(err);
						});
					}
					resolve(res.registered);
				}).catch((err: any) => {
					resolve(err);
				})
			}
		});
	})
}
// 授权
function isAuthorization(to: any, from: any, next: any) {
	if (to.name === 'authorization') return;
	if (!to.meta.authorization) return;
	return $http.get({
		url: $api.getUserInfo,
	}).then((res: any) => {
		if (res) {
			Store.commit(`base/${types.SET_USERINFO}`, res.info);
		}
		if (!res.info.phone) {
			next({
				path: '/packages/authorization/authorization',
				query: { to: to.path, data: to.query, navType: to.NAVTYPE },
				NAVTYPE: 'push'
			});
			return true;
		}
	}, (err: any) => {
		console.log(err);
	})
}
//路由守卫
router.afterEach((to, from) => { });

export { router, RouterMount };

// export default router;
